--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity fmc_hv_2ch_if is
  generic(
    --synthesis or simuation mode
    g_SYNTHESIS                : integer  := 1;     -- 1=true / 0=false
    --main clock frequency     
    g_CLOCK_FREQ               : positive := 100E6  -- Main clk freq in Hz
  );
  port (
    --Clock and reset
    rst_ir                     : in    std_logic; 
    clk_ik                     : in    std_logic; 
    cen_ie                     : in    std_logic; 
    --CTRL/STATUS
    present_o                  : out   std_logic; 
    enable_ctrl_i              : in    std_logic; 
    enable_stat_o              : out   std_logic; 
    power_good_o               : out   std_logic; 
    --PWM
    fmc_HV1_pwm_ena_i          : in    std_logic;                      -- duty cycle 0%
    fmc_HV1_pwm_duty_cycle_ib8 : in    std_logic_vector(7 downto 0);   -- 0= 0.4% 1/256 cycles   1= 100% 256/256 cycles
    fmc_HV1_pwm_clk_div_ib32   : in    std_logic_vector(31 downto 0);  -- pwm frequency
    fmc_HV1_oe_i               : in    std_logic; 
    fmc_HV2_pwm_ena_i          : in    std_logic;                      -- duty cycle 0%
    fmc_HV2_pwm_duty_cycle_ib8 : in    std_logic_vector(7 downto 0);   -- 0= 0.4% 1/256 cycles   1= 100% 256/256 cycles
    fmc_HV2_pwm_clk_div_ib32   : in    std_logic_vector(31 downto 0);  -- pwm frequency
    fmc_HV2_oe_i               : in    std_logic; 
    fmc_LV_pwm_ena_i           : in    std_logic;                      -- duty cycle 0%
    fmc_LV_pwm_duty_cycle_ib8  : in    std_logic_vector(7 downto 0);   -- 0= 0.4% 1/256 cycles   1= 100% 256/256 cycles
    fmc_LV_pwm_clk_div_ib32    : in    std_logic_vector(31 downto 0);  -- pwm frequency
    fmc_LV_oe_i                : in    std_logic; 
    --DAC1
    DAC1_HV1_wr_i              : in    std_logic; 
    DAC1_HV1_ib16              : in    std_logic_vector(15 downto 0); 
    DAC1_HV1_busy_o            : out   std_logic; 
    DAC1_TP3_wr_i              : in    std_logic; 
    DAC1_TP3_ib16              : in    std_logic_vector(15 downto 0); 
    DAC1_TP3_busy_o            : out   std_logic; 
    --DAC2
    DAC2_HV2_wr_i              : in    std_logic; 
    DAC2_HV2_ib16              : in    std_logic_vector(15 downto 0); 
    DAC2_HV2_busy_o            : out   std_logic; 
    DAC2_LV_wr_i               : in    std_logic; 
    DAC2_LV_ib16               : in    std_logic_vector(15 downto 0);
    DAC2_LV_busy_o             : out   std_logic; 
    --ADC1
    ADC1_ch_ob25               : out   std_logic_vector(24 downto 0); 
    ADC1_ch_valid_o            : out   std_logic; 
    ADC1_temp_rd_i             : in    std_logic;
    ADC1_temp_ob25             : out   std_logic_vector(24 downto 0); 
    ADC1_temp_rd_valid_o       : out   std_logic; 
    ADC1_temp_busy_o           : out   std_logic; 
    --ADC2
    ADC2_ch_ob25               : out   std_logic_vector(24 downto 0); 
    ADC2_ch_valid_o            : out   std_logic; 
    ADC2_temp_rd_i             : in    std_logic;
    ADC2_temp_ob25             : out   std_logic_vector(24 downto 0); 
    ADC2_temp_rd_valid_o       : out   std_logic; 
    ADC2_temp_busy_o           : out   std_logic; 
    --ADC3
    ADC3_LV_volt_valid_o       : out   std_logic;
    ADC3_LV_volt_ob16          : out   std_logic_vector(15 downto 0);
    ADC3_HV1_curr_valid_o      : out   std_logic;
    ADC3_HV1_curr_ob16         : out   std_logic_vector(15 downto 0); 
    ADC3_HV2_curr_valid_o      : out   std_logic;
    ADC3_HV2_curr_ob16         : out   std_logic_vector(15 downto 0); 
    ADC3_LV_curr_valid_o       : out   std_logic;
    ADC3_LV_curr_ob16          : out   std_logic_vector(15 downto 0); 
    ADC3_4ch_temp_valid_o      : out   std_logic;
    ADC3_4ch_temp_ob16         : out   std_logic_vector(15 downto 0); 
    -- EEPROM WB
    eeprom_wb_cyc_i            : in    std_logic;
    eeprom_wb_stb_i            : in    std_logic;
    eeprom_wb_adr_ib2          : in    std_logic_vector(1 downto 0);
    eeprom_wb_sel_ib4          : in    std_logic_vector(3 downto 0);
    eeprom_wb_we_i             : in    std_logic;
    eeprom_wb_dat_mosi_ib32    : in    std_logic_vector(31 downto 0);
    eeprom_wb_dat_miso_ob32    : out   std_logic_vector(31 downto 0);
    eeprom_wb_ack_o            : out   std_logic;
    eeprom_wb_stall_o          : out   std_logic;
    ----------------------
    --FMC I/F
    fmc_present_i              : in    std_logic; 
    fmc_power_good_i           : in    std_logic; 
    fmc_enable_ctrl_o          : out   std_logic;
    fmc_enable_stat_i          : in    std_logic; 
    fmc_HV1_outena_o           : out   std_logic;  --tri-state output buffer
    fmc_HV2_outena_o           : out   std_logic;  --tri-state output buffer
    fmc_LV_outena_o            : out   std_logic;  --tri-state output buffer
    fmc_DAC1_sync_no           : out   std_logic;
    fmc_DAC1_clk_o             : out   std_logic; 
    fmc_DAC1_din_o             : out   std_logic; 
    fmc_DAC2_sync_no           : out   std_logic; 
    fmc_DAC2_clk_o             : out   std_logic; 
    fmc_DAC2_din_o             : out   std_logic; 
    fmc_ADC1_Vfb_clk_o         : out   std_logic; 
    fmc_ADC1_Vfb_cs_o          : out   std_logic;  
    fmc_ADC1_Vfb_mosi_o        : out   std_logic; 
    fmc_ADC1_Vfb_miso_i        : in    std_logic; 
    fmc_ADC2_Vfb_clk_o         : out   std_logic; 
    fmc_ADC2_Vfb_cs_o          : out   std_logic;  
    fmc_ADC2_Vfb_mosi_o        : out   std_logic; 
    fmc_ADC2_Vfb_miso_i        : in    std_logic; 
    fmc_ADC_sck_o              : out   std_logic; 
    fmc_ADC_cnv_o              : out   std_logic; 
    fmc_ADC_din_o              : out   std_logic; 
    fmc_ADC_sdo_i              : in    std_logic;
    fmc_i2c_prom_scl_iok       : inout std_logic; 
    fmc_i2c_prom_sda_io        : inout std_logic 
  );
end entity ;




------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of fmc_hv_2ch_if is
  
  --5us
  constant c_FMC_SETTLING_TIME : unsigned(9 downto 0) := to_unsigned(5000/(1E9/g_CLOCK_FREQ), 10); --Number of clock cycles in 5us
  signal settling_timeout      : std_logic;
  signal settling_timer        : unsigned(9 downto 0);

  --LENGTHS
  constant c_PWM_LEN           : positive range 1 to 32:= 8; 
  constant c_CLK_DIV_CNT_LEN   : positive range 1 to 32:= 32;

  --EEPROM I2C freq
  constant c_I2C_FREQ          : positive := 100E3;                                       -- SCL 100kHz or 10us (high > 4us, low>4.7us) 
  constant c_I2C_SCL_RATE      : positive := (g_CLOCK_FREQ/c_I2C_FREQ)/2;                 -- 100MHz to 100kHz, divided by 2 for half period
  constant c_I2C_SCL_RATE_B10  : unsigned(9 downto 0) := to_unsigned(c_I2C_SCL_RATE, 10); -- 1000/2=500 clock cycles (on 10bits)

  --PWM  
  signal fmc_HV1_active        : std_logic;
  signal fmc_HV2_active        : std_logic;
  signal fmc_LV_active         : std_logic;
  
  --FMC enable
  signal fmc_enable_stat       : std_logic;

begin 


  -----------
  -- mezzanine presence
  -----------
  present_o <= fmc_present_i;     --11k pull-up on VFC pin H2 (ERROR!!! on mezanine schematics: not strong pull down <11kohms)
  
  -----------
  -- mezzanine voltage enable (3V3 and 12V)
  -----------
  fmc_enable_ctrl_o <= enable_ctrl_i;
  fmc_enable_stat   <= fmc_enable_stat_i and settling_timeout; --enable local SPI links
  enable_stat_o     <= fmc_enable_stat;
  
  -----------
  -- FMC DAC/ADC/EEPROm settling timer (~5us)
  -----------
  p_settling_timeout : process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then 
      settling_timeout <= '0';
      settling_timer   <= c_FMC_SETTLING_TIME;
      
    elsif rising_edge(clk_ik) then
      --FMC power down
      if enable_ctrl_i='0' then
        settling_timeout <= '0';
        settling_timer   <= c_FMC_SETTLING_TIME;
      --Settling time
      else
        if settling_timer=0 then
          settling_timeout <= '1';
        else
          settling_timer <= settling_timer-1;
        end if;
      end if;
    end if;
  end process;
  
  -----------
  -- power good from mezzanine
  -----------
  power_good_o <= fmc_power_good_i; --11k pull-up on VFC pin F1 (no row F on LPC)
  
  
  -----------
  -- Voltage output Enable (tri-state output buffer)
  -----------
  fmc_HV1_outena_o <= fmc_HV1_active when fmc_HV1_oe_i='1' else 'Z';
  fmc_HV2_outena_o <= fmc_HV2_active when fmc_HV2_oe_i='1' else 'Z';
  fmc_LV_outena_o  <= fmc_LV_active  when  fmc_LV_oe_i='1' else 'Z';


  -----------
  -- HV1 PWM
  -----------
  i_pwm_HV1: entity work.pwm
  generic map(
    g_PWM_LEN         => c_PWM_LEN,              -- positive :=8; --gives the resolution (8b => 255 possible ratio)
    g_CLK_DIV_CNT_LEN => c_CLK_DIV_CNT_LEN       -- positive :=1  --gives the cycle period (frequency)
  )
  port map(
    --clock and reset
    rst_ir        => rst_ir,                     -- in  std_logic; 
    clk_ik        => clk_ik,                     -- in  std_logic; 
    cen_ie        => cen_ie,                     -- in  std_logic; 
    --input        
    enable_i      => fmc_HV1_pwm_ena_i,          -- in  std_logic; 
    duty_cycle_ib => fmc_HV1_pwm_duty_cycle_ib8, -- in  std_logic_vector(g_PWM_LEN-1 downto 0); 
    clk_div_ib    => fmc_HV1_pwm_clk_div_ib32,   -- in  std_logic_vector(g_CLK_DIV_CNT_LEN-1 downto 0); 
    --output        
    pwm_o         => fmc_HV1_active              -- out std_logic
  );

  
  -----------
  -- HV2 PWM
  -----------
  i_pwm_HV2: entity work.pwm
  generic map(
    g_PWM_LEN         => c_PWM_LEN,              -- positive :=8; --gives the resolution (8b => 255 possible ratio)
    g_CLK_DIV_CNT_LEN => c_CLK_DIV_CNT_LEN       -- positive :=1  --gives the cycle period (frequency)
  )
  port map(
    --clock and reset
    rst_ir        => rst_ir,                     -- in  std_logic; 
    clk_ik        => clk_ik,                     -- in  std_logic; 
    cen_ie        => cen_ie,                     -- in  std_logic; 
    --input        
    enable_i      => fmc_HV2_pwm_ena_i,          -- in  std_logic; 
    duty_cycle_ib => fmc_HV2_pwm_duty_cycle_ib8, -- in  std_logic_vector(g_PWM_LEN-1 downto 0); 
    clk_div_ib    => fmc_HV2_pwm_clk_div_ib32,   -- in  std_logic_vector(g_CLK_DIV_CNT_LEN-1 downto 0); 
    --output        
    pwm_o         => fmc_HV2_active              -- out std_logic
  );
  
  -----------
  -- LV PWM
  -----------
  i_pwm_LV: entity work.pwm
  generic map(
    g_PWM_LEN         => c_PWM_LEN,              -- positive :=8; --gives the resolution (8b => 255 possible ratio)
    g_CLK_DIV_CNT_LEN => c_CLK_DIV_CNT_LEN       -- positive :=1  --gives the cycle period (frequency)
  )
  port map(
    --clock and reset
    rst_ir        => rst_ir,                    -- in  std_logic; 
    clk_ik        => clk_ik,                    -- in  std_logic; 
    cen_ie        => cen_ie,                    -- in  std_logic; 
    --input        
    enable_i      => fmc_LV_pwm_ena_i,          -- in  std_logic; 
    duty_cycle_ib => fmc_LV_pwm_duty_cycle_ib8, -- in  std_logic_vector(g_PWM_LEN-1 downto 0); 
    clk_div_ib    => fmc_LV_pwm_clk_div_ib32,   -- in  std_logic_vector(g_CLK_DIV_CNT_LEN-1 downto 0); 
    --output        
    pwm_o         => fmc_LV_active              -- out std_logic
  );

  -----------
  -- HV1/TP3 DAC SPI (AD5663) 
  -----------
  i_dac_ad5663_1: entity work.dac_ad5663armz
  generic map(
    g_CLOCK_FREQ => g_CLOCK_FREQ -- positive := 100
  )
  port map (
    --clk & rst
    rst_ir      => rst_ir,           -- in  std_logic; 
    clk_ik      => clk_ik,           -- in  std_logic; 
    cen_ie      => cen_ie,           -- in  std_logic; 
    --FMC enable
    enable_i    => fmc_enable_stat,  -- in  std_logic; 
    --CTRL
    chA_wr_i    => DAC1_HV1_wr_i,    -- in  std_logic; 
    chA_ib16    => DAC1_HV1_ib16,    -- in  std_logic_vector(15 downto 0); 
    chA_busy_o  => DAC1_HV1_busy_o,  -- out std_logic; 
    chB_wr_i    => DAC1_TP3_wr_i,    -- in  std_logic; 
    chB_ib16    => DAC1_TP3_ib16,    -- in  std_logic_vector(15 downto 0); 
    chB_busy_o  => DAC1_TP3_busy_o,  -- out std_logic; 
    --SPI output 
    SPI_sync_no => fmc_DAC1_sync_no, -- out std_logic;
    SPI_clk_o   => fmc_DAC1_clk_o,   -- out std_logic; 
    SPI_mosi_o  => fmc_DAC1_din_o    -- out std_logic 
  );


  -----------
  -- HV2/LV DAC SPI (AD5663)
  -----------
  i_dac_ad5663_2: entity work.dac_ad5663armz
  generic map(
    g_CLOCK_FREQ => g_CLOCK_FREQ -- positive := 100E6 
  )
  port map (
    --clk & rst
    rst_ir      => rst_ir,           -- in  std_logic; 
    clk_ik      => clk_ik,           -- in  std_logic; 
    cen_ie      => cen_ie,           -- in  std_logic; 
    --FMC enable
    enable_i    => fmc_enable_stat,  -- in  std_logic; 
    --CTRL
    chA_wr_i    => DAC2_HV2_wr_i,    -- in  std_logic; 
    chA_ib16    => DAC2_HV2_ib16,    -- in  std_logic_vector(15 downto 0); 
    chA_busy_o  => DAC2_HV2_busy_o,  -- out std_logic; 
    chB_wr_i    => DAC2_LV_wr_i,     -- in  std_logic; 
    chB_ib16    => DAC2_LV_ib16,     -- in  std_logic_vector(15 downto 0); 
    chB_busy_o  => DAC2_LV_busy_o,   -- out std_logic; 
    --SPI output 
    SPI_sync_no => fmc_DAC2_sync_no, -- out std_logic;
    SPI_clk_o   => fmc_DAC2_clk_o,   -- out std_logic; 
    SPI_mosi_o  => fmc_DAC2_din_o    -- out std_logic 
  );


  -----------
  -- HV1 ADC SPI (LTC2484)
  -----------
  i_adc_hv_ltc2484_1: entity work.adc_hv_ltc2484
  generic map(
    --synthesis or simuation mode
    g_SYNTHESIS               => g_SYNTHESIS,    -- integer  := 1;     -- 1=true / 0=false
    --main clock frequency
    g_CLOCK_FREQ              => g_CLOCK_FREQ,   -- positive := 100E6  -- Main clk freq in MHz
    -- Refresh period
    g_WAIT_GAP_BEFORE_REFRESH => 0               -- integer  := 0;     -- clock cycles between the next conversion         
  )
  port map(
    --clk & rst
    rst_ir          => rst_ir,                   -- in  std_logic; 
    clk_ik          => clk_ik,                   -- in  std_logic; 
    cen_ie          => cen_ie,                   -- in  std_logic; 
    --FMC enable                                 
    enable_i        => fmc_enable_stat,          -- in  std_logic; 
    --ADC CTRL                                   
    ch_ob25         => ADC1_ch_ob25,             -- out std_logic_vector(24 downto 0); 
    ch_valid_o      => ADC1_ch_valid_o,          -- out std_logic; 
    --Temperature request (give the last value and trigger a new conversion)
    temp_rd_i       => ADC1_temp_rd_i,           -- in  std_logic;
    temp_ob25       => ADC1_temp_ob25,           -- out std_logic_vector(24 downto 0);  downto 0); 
    temp_rd_valid_o => ADC1_temp_rd_valid_o,     -- out std_logic; 
    temp_busy_o     => ADC1_temp_busy_o,         -- out std_logic; 
    --SPI output                                 
    SPI_sync_no     => fmc_ADC1_Vfb_cs_o,        -- out std_logic;  --1k pull-down on FMC
    SPI_clk_o       => fmc_ADC1_Vfb_clk_o,       -- out std_logic; 
    SPI_mosi_o      => fmc_ADC1_Vfb_mosi_o,      -- out std_logic 
    SPI_miso_o      => fmc_ADC1_Vfb_miso_i       -- in  std_logic 
  );


  -----------
  -- HV2 ADC SPI (LTC2484)
  -----------
  i_adc_hv_ltc2484_2: entity work.adc_hv_ltc2484
  generic map(
    --synthesis or simuation mode
    g_SYNTHESIS               => g_SYNTHESIS,    -- integer  := 1;     -- 1=true / 0=false
    --main clock frequency
    g_CLOCK_FREQ              => g_CLOCK_FREQ,   -- positive := 100E6  -- Main clk freq in MHz
    -- Refresh period
    g_WAIT_GAP_BEFORE_REFRESH => 0               -- integer  := 0;     -- clock cycles between the next conversion         
  )
  port map(
    --clk & rst
    rst_ir          => rst_ir,                   -- in  std_logic; 
    clk_ik          => clk_ik,                   -- in  std_logic; 
    cen_ie          => cen_ie,                   -- in  std_logic; 
    --FMC enable                                 
    enable_i        => fmc_enable_stat,          -- in  std_logic;  
    --ADC CTRL                                   
    ch_ob25         => ADC2_ch_ob25,             -- out std_logic_vector(24 downto 0); 
    ch_valid_o      => ADC2_ch_valid_o,          -- out std_logic; 
    --Temperature request (give the last value and trigger a new conversion)
    temp_rd_i       => ADC2_temp_rd_i,           -- in  std_logic;
    temp_ob25       => ADC2_temp_ob25,           -- out std_logic_vector(24 downto 0);  downto 0); 
    temp_rd_valid_o => ADC2_temp_rd_valid_o,     -- out std_logic; 
    temp_busy_o     => ADC2_temp_busy_o,         -- out std_logic; 
    --SPI output                                 
    SPI_sync_no     => fmc_ADC2_Vfb_cs_o,        -- out std_logic;  --1k pull-down on FMC
    SPI_clk_o       => fmc_ADC2_Vfb_clk_o,       -- out std_logic; 
    SPI_mosi_o      => fmc_ADC2_Vfb_mosi_o,      -- out std_logic 
    SPI_miso_o      => fmc_ADC2_Vfb_miso_i       -- in  std_logic 
  );
  


  -----------
  -- LV ADC SPI (AD7682) 
  -----------
  i_adc3_4ch_ad7682: entity work.adc_4ch_ad7682
  generic map(
    --synthesis or simuation mode
    g_SYNTHESIS                => g_SYNTHESIS,      -- integer  := 1;   -- 1=true / 0=false 
    --main clock frequency                          
    g_CLOCK_FREQ               => g_CLOCK_FREQ,     -- positive := 100; -- Main clk freq in MHz
    -- Refresh period                               
    g_WAIT_GAP_BEFORE_REFRESH  => 100000   --1ms    -- integer  := 0    -- clock cycles between the next conversion         
  )
  port map(
    --clk & rst
    rst_ir                => rst_ir,                -- in     std_logic; 
    clk_ik                => clk_ik,                -- in     std_logic; 
    cen_ie                => cen_ie,                -- in     std_logic; 
    --FMC enable                            
    enable_i              => fmc_enable_stat,       -- in     std_logic; 
    --ADC CTRL            
    ADC3_LV_volt_valid_o  => ADC3_LV_volt_valid_o,  -- out   std_logic;
    ADC3_LV_volt_ob16     => ADC3_LV_volt_ob16,     -- out   std_logic_vector(15 downto 0);
    ADC3_HV1_curr_valid_o => ADC3_HV1_curr_valid_o, -- out   std_logic;
    ADC3_HV1_curr_ob16    => ADC3_HV1_curr_ob16,    -- out   std_logic_vector(15 downto 0); 
    ADC3_HV2_curr_valid_o => ADC3_HV2_curr_valid_o, -- out   std_logic;
    ADC3_HV2_curr_ob16    => ADC3_HV2_curr_ob16,    -- out   std_logic_vector(15 downto 0); 
    ADC3_LV_curr_valid_o  => ADC3_LV_curr_valid_o,  -- out   std_logic;
    ADC3_LV_curr_ob16     => ADC3_LV_curr_ob16,     -- out   std_logic_vector(15 downto 0); 
    ADC3_4ch_temp_valid_o => ADC3_4ch_temp_valid_o, -- out   std_logic;
    ADC3_4ch_temp_ob16    => ADC3_4ch_temp_ob16,    -- out   std_logic_vector(15 downto 0); 
    --SPI output    
    SPI_sync_no           => fmc_ADC_cnv_o,         -- out    std_logic;
    SPI_clk_o             => fmc_ADC_sck_o,         -- out    std_logic; 
    SPI_mosi_o            => fmc_ADC_din_o,         -- out    std_logic;
    SPI_miso_o            => fmc_ADC_sdo_i          -- in     std_logic
  );

  -----------
  -- I2C PROM (always active, EEPROM on P3V3AUX)
  -----------
  i_eeprom_i2c_mst: entity work.i2c_master_wb
  generic map(
    -- synthesis or simuation mode
    g_SYNTHESIS => g_SYNTHESIS,                   -- integer := 1; -- 1=true / 0=false
    -- length of the I2C cycle
    g_CYCLE_LEN => to_integer(c_I2C_SCL_RATE_B10) -- integer(9 downto 0);
  )  
  port map(
    -- Clock and reset
    rst_ir      => rst_ir,                        -- in    std_logic;
    clk_ik      => clk_ik,                        -- in    std_logic;
    cen_ie      => cen_ie,                        -- in    std_logic;
    --WB                                          
    wb_cyc_i    => eeprom_wb_cyc_i,               -- in    std_logic;
    wb_stb_i    => eeprom_wb_stb_i,               -- in    std_logic;
    wb_we_i     => eeprom_wb_we_i,                -- in    std_logic;
    wb_adr_ib2  => eeprom_wb_adr_ib2,             -- in    std_logic_vector(1 downto 0);
    wb_sel_ib4  => eeprom_wb_sel_ib4,             -- in    std_logic_vector(3 downto 0);
    wb_dat_ib32 => eeprom_wb_dat_mosi_ib32,       -- in    std_logic_vector(31 downto 0);
    wb_dat_ob32 => eeprom_wb_dat_miso_ob32,       -- out   std_logic_vector(31 downto 0);
    wb_ack_o    => eeprom_wb_ack_o,               -- out   std_logic;
    wb_stall_o  => eeprom_wb_stall_o,             -- out   std_logic;
    --I2C                                         
    i2c_scl_io  => fmc_i2c_prom_scl_iok,          -- inout std_logic;
    i2c_sda_io  => fmc_i2c_prom_sda_io            -- inout std_logic;
  );
  
end architecture;