--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity fmc_hv_2ch_regulator is
  generic(
    --synthesis or simuation mode
    g_SYNTHESIS          : integer  := 1  -- 1=true / 0=false
  );
  port (
    --CLK/RST
    rst_ir               : in    std_logic; 
    clk_ik               : in    std_logic; 
    cen_ie               : in    std_logic; 
    --enable inputs  
    HV1_regulator_ena_i  : in    std_logic; 
    HV2_regulator_ena_i  : in    std_logic; 
    LV_regulator_ena_i   : in    std_logic; 
    --Voltage commands inputs
    HV1_volt_cmd_wr_i    : in    std_logic; 
    HV1_volt_cmd_ib16    : in    std_logic_vector(15 downto 0); 
    HV2_volt_cmd_wr_i    : in    std_logic; 
    HV2_volt_cmd_ib16    : in    std_logic_vector(15 downto 0); 
    LV_volt_cmd_wr_i     : in    std_logic; 
    LV_volt_cmd_ib16     : in    std_logic_vector(15 downto 0); 
    --Master WishBone
    Cyc_i                : in    std_logic; 
    Stb_i                : in    std_logic; 
    Adr_ib               : in    std_logic_vector(31 downto 0); 
    Sel_ib               : in    std_logic_vector(3  downto 0); 
    We_i                 : in    std_logic; 
    DatMoSi_ib           : in    std_logic_vector(31 downto 0); 
    DatMiSo_oqb          : out   std_logic_vector(31 downto 0); 
    Ack_oq               : out   std_logic; 
    Stall_o              : out   std_logic; 
    --HV1
    HV1_DAC_wr_o         : out   std_logic; 
    HV1_DAC_ob16         : out   std_logic_vector(15 downto 0); 
    HV1_ADC_volt_valid_i : in    std_logic;
    HV1_ADC_volt_ib25    : in    std_logic_vector(24 downto 0); 
    HV1_ADC_curr_valid_i : in    std_logic;
    HV1_ADC_curr_ib16    : in    std_logic_vector(15 downto 0); 
    --HV2 
    HV2_DAC_wr_o         : out   std_logic; 
    HV2_DAC_ob16         : out   std_logic_vector(15 downto 0); 
    HV2_ADC_volt_valid_i : in    std_logic;
    HV2_ADC_volt_ib25    : in    std_logic_vector(24 downto 0); 
    HV2_ADC_curr_valid_i : in    std_logic;
    HV2_ADC_curr_ib16    : in    std_logic_vector(15 downto 0); 
    --LV
    LV_DAC_wr_o          : out   std_logic; 
    LV_DAC_ob16          : out   std_logic_vector(15 downto 0); 
    LV_ADC_volt_valid_i  : in    std_logic;
    LV_ADC_volt_ib16     : in    std_logic_vector(15 downto 0); 
    LV_ADC_curr_valid_i  : in    std_logic;
    LV_ADC_curr_ib16     : in    std_logic_vector(15 downto 0) 
  );
end entity ;


------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of fmc_hv_2ch_regulator is
  

begin 

  -----------
  -- Configuration register via WB
  -----------
  i_32_regul_conf_reg: entity work.wb_32out_reg
  generic map(
    g_ADDR_WIDTH        => 5,           -- integer  := 5; 
    g_DATA_WIDTH        => 32,          -- integer  := 32;
    g_reg0_default      => x"00000000", -- std_logic_vector;
    g_reg0_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg1_default      => x"00000000", -- std_logic_vector;
    g_reg1_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg2_default      => x"00000000", -- std_logic_vector;
    g_reg2_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg3_default      => x"00000000", -- std_logic_vector;
    g_reg3_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg4_default      => x"00000000", -- std_logic_vector;
    g_reg4_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg5_default      => x"00000000", -- std_logic_vector;
    g_reg5_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg6_default      => x"00000000", -- std_logic_vector;
    g_reg6_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg7_default      => x"00000000", -- std_logic_vector;
    g_reg7_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg8_default      => x"00000000", -- std_logic_vector;
    g_reg8_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg9_default      => x"00000000", -- std_logic_vector; 
    g_reg9_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;    
    g_reg10_default     => x"00000000", -- std_logic_vector;
    g_reg10_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg11_default     => x"00000000", -- std_logic_vector;
    g_reg11_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg12_default     => x"00000000", -- std_logic_vector;
    g_reg12_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg13_default     => x"00000000", -- std_logic_vector;
    g_reg13_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg14_default     => x"00000000", -- std_logic_vector;
    g_reg14_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg15_default     => x"00000000", -- std_logic_vector;
    g_reg15_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg16_default     => x"00000000", -- std_logic_vector;
    g_reg16_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg17_default     => x"00000000", -- std_logic_vector;
    g_reg17_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg18_default     => x"00000000", -- std_logic_vector;
    g_reg18_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg19_default     => x"00000000", -- std_logic_vector;
    g_reg19_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg20_default     => x"00000000", -- std_logic_vector;
    g_reg20_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg21_default     => x"00000000", -- std_logic_vector;
    g_reg21_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg22_default     => x"00000000", -- std_logic_vector;
    g_reg22_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg23_default     => x"00000000", -- std_logic_vector;
    g_reg23_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg24_default     => x"00000000", -- std_logic_vector;
    g_reg24_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg25_default     => x"00000000", -- std_logic_vector;
    g_reg25_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg26_default     => x"00000000", -- std_logic_vector;
    g_reg26_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg27_default     => x"00000000", -- std_logic_vector;
    g_reg27_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg28_default     => x"00000000", -- std_logic_vector;
    g_reg28_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg29_default     => x"00000000", -- std_logic_vector;
    g_reg29_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg30_default     => x"00000000", -- std_logic_vector;
    g_reg30_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg31_default     => x"00000000", -- std_logic_vector;
    g_reg31_autoclrmask => x"FFFFFFFF"  -- std_logic_vector
  )
  port map(
    --CLK/RST
    rst_ir       => rst_ir,             -- in    std_logic; 
    clk_ik       => clk_ik,             -- in    std_logic; 
    cen_ie       => cen_ie,             -- in    std_logic; 
    --Master WishBone            
    Cyc_i        => Cyc_i,              -- in    std_logic; 
    Stb_i        => Stb_i,              -- in    std_logic; 
    Adr_ib       => Adr_ib(4 downto 0), -- in    std_logic_vector(g_ADDR_WIDTH-1 downto 0); 
    Sel_ib       => Sel_ib,             -- in    std_logic_vector(g_ADDR_WIDTH/8-1 downto 0); 
    We_i         => We_i,               -- in    std_logic; 
    DatMoSi_ib   => DatMoSi_ib,         -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    DatMiSo_oqb  => DatMiSo_oqb,        -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    Ack_oq       => Ack_oq,             -- out   std_logic; 
    Stall_o      => Stall_o,            -- out   std_logic; 
    -- update strobe                    
    reg_wr_ob32  => open,               -- out   std_logic_vector(31 downto 0); 
    --Registers                                        
    reg0_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg1_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg2_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg3_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg4_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg5_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg6_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg7_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg8_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg9_ob      => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg10_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg11_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg12_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg13_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg14_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg15_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg16_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg17_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg18_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg19_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg20_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg21_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg22_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg23_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg24_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg25_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg26_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg27_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg28_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg29_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg30_ob     => open,               -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg31_ob     => open                -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0)
  );

  -----------
  -- TO BE DONE BY JEMERY (signal forwarding for now)
  -----------
  --HV1 
  HV1_DAC_wr_o   <= HV1_volt_cmd_wr_i;
  HV1_DAC_ob16   <= HV1_volt_cmd_ib16; 
  --HV2 
  HV2_DAC_wr_o   <= HV2_volt_cmd_wr_i;
  HV2_DAC_ob16   <= HV2_volt_cmd_ib16; 
  --LV   
  LV_DAC_wr_o    <= LV_volt_cmd_wr_i;
  LV_DAC_ob16    <= LV_volt_cmd_ib16; 
  
end architecture;
