--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity wb_32in_reg is
  generic(
    g_ADDR_WIDTH : integer  := 5; 
    g_DATA_WIDTH : integer  := 32
  );
  port (
    --CLK/RST
    rst_ir       : in    std_logic; 
    clk_ik       : in    std_logic; 
    cen_ie       : in    std_logic; 
    --Master WishBone
    Cyc_i        : in    std_logic; 
    Stb_i        : in    std_logic; 
    Adr_ib       : in    std_logic_vector(g_ADDR_WIDTH-1 downto 0); 
    Sel_ib       : in    std_logic_vector(g_DATA_WIDTH/8-1 downto 0); 
    We_i         : in    std_logic; 
    DatMoSi_ib   : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    DatMiSo_oqb  : out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    Ack_oq       : out   std_logic; 
    Stall_o      : out   std_logic; 
    --Registers
    reg0_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg1_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg2_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg3_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg4_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg5_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg6_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg7_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg8_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg9_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg10_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg11_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg12_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg13_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg14_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg15_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg16_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg17_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg18_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg19_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg20_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg21_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg22_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg23_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg24_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg25_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg26_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg27_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg28_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg29_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg30_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg31_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0)
  );
end entity;




------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of wb_32in_reg is
  
  signal Ack_q : std_logic;

begin 
  
  -----------------------
  -- Address decoding
  -----------------------
  p_addr_decod: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      DatMiSo_oqb <= (others=>'0');
      Ack_q       <= '0';
      Stall_o     <= '0';

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        
        Ack_q    <= Stb_i AND Cyc_i AND not(Ack_q);
        Stall_o  <= '0';
        
        if Ack_q = '0' then 
             if unsigned(Adr_ib) = to_unsigned( 0,g_ADDR_WIDTH) then DatMiSo_oqb <= reg0_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 1,g_ADDR_WIDTH) then DatMiSo_oqb <= reg1_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 2,g_ADDR_WIDTH) then DatMiSo_oqb <= reg2_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 3,g_ADDR_WIDTH) then DatMiSo_oqb <= reg3_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 4,g_ADDR_WIDTH) then DatMiSo_oqb <= reg4_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 5,g_ADDR_WIDTH) then DatMiSo_oqb <= reg5_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 6,g_ADDR_WIDTH) then DatMiSo_oqb <= reg6_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 7,g_ADDR_WIDTH) then DatMiSo_oqb <= reg7_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 8,g_ADDR_WIDTH) then DatMiSo_oqb <= reg8_ib ;
          elsif unsigned(Adr_ib) = to_unsigned( 9,g_ADDR_WIDTH) then DatMiSo_oqb <= reg9_ib ;
          elsif unsigned(Adr_ib) = to_unsigned(10,g_ADDR_WIDTH) then DatMiSo_oqb <= reg10_ib;
          elsif unsigned(Adr_ib) = to_unsigned(11,g_ADDR_WIDTH) then DatMiSo_oqb <= reg11_ib;
          elsif unsigned(Adr_ib) = to_unsigned(12,g_ADDR_WIDTH) then DatMiSo_oqb <= reg12_ib;
          elsif unsigned(Adr_ib) = to_unsigned(13,g_ADDR_WIDTH) then DatMiSo_oqb <= reg13_ib;
          elsif unsigned(Adr_ib) = to_unsigned(14,g_ADDR_WIDTH) then DatMiSo_oqb <= reg14_ib;
          elsif unsigned(Adr_ib) = to_unsigned(15,g_ADDR_WIDTH) then DatMiSo_oqb <= reg15_ib;
          elsif unsigned(Adr_ib) = to_unsigned(16,g_ADDR_WIDTH) then DatMiSo_oqb <= reg16_ib;
          elsif unsigned(Adr_ib) = to_unsigned(17,g_ADDR_WIDTH) then DatMiSo_oqb <= reg17_ib;
          elsif unsigned(Adr_ib) = to_unsigned(18,g_ADDR_WIDTH) then DatMiSo_oqb <= reg18_ib;
          elsif unsigned(Adr_ib) = to_unsigned(19,g_ADDR_WIDTH) then DatMiSo_oqb <= reg19_ib;
          elsif unsigned(Adr_ib) = to_unsigned(20,g_ADDR_WIDTH) then DatMiSo_oqb <= reg20_ib;
          elsif unsigned(Adr_ib) = to_unsigned(21,g_ADDR_WIDTH) then DatMiSo_oqb <= reg21_ib;
          elsif unsigned(Adr_ib) = to_unsigned(22,g_ADDR_WIDTH) then DatMiSo_oqb <= reg22_ib;
          elsif unsigned(Adr_ib) = to_unsigned(23,g_ADDR_WIDTH) then DatMiSo_oqb <= reg23_ib;
          elsif unsigned(Adr_ib) = to_unsigned(24,g_ADDR_WIDTH) then DatMiSo_oqb <= reg24_ib;
          elsif unsigned(Adr_ib) = to_unsigned(25,g_ADDR_WIDTH) then DatMiSo_oqb <= reg25_ib;
          elsif unsigned(Adr_ib) = to_unsigned(26,g_ADDR_WIDTH) then DatMiSo_oqb <= reg26_ib;
          elsif unsigned(Adr_ib) = to_unsigned(27,g_ADDR_WIDTH) then DatMiSo_oqb <= reg27_ib;
          elsif unsigned(Adr_ib) = to_unsigned(28,g_ADDR_WIDTH) then DatMiSo_oqb <= reg28_ib;
          elsif unsigned(Adr_ib) = to_unsigned(29,g_ADDR_WIDTH) then DatMiSo_oqb <= reg29_ib;
          elsif unsigned(Adr_ib) = to_unsigned(30,g_ADDR_WIDTH) then DatMiSo_oqb <= reg30_ib;
          elsif unsigned(Adr_ib) = to_unsigned(31,g_ADDR_WIDTH) then DatMiSo_oqb <= reg31_ib;
          else DatMiSo_oqb <= x"ABADCAFE";
          end if;
        end if;
      end if;
    end if;
  end process;
  
  --output
  Ack_oq <= Ack_q AND Stb_i;

end architecture;